__all__ = ("initialize_twitter", "login_twitter", "logout_twitter")


def initialize_twitter(
    success_listener, error_listener, client_id=None, client_secret=None
):
    raise NotImplementedError


def login_twitter():
    raise NotImplementedError


def logout_twitter():
    raise NotImplementedError



from android.activity import bind as result_bind
from jnius import PythonJavaClass, autoclass, java_method
from kivy.logger import Logger

FirebaseAuth = autoclass("com.google.firebase.auth.FirebaseAuth")
OAuthProvider = autoclass("com.google.firebase.auth.OAuthProvider")

PythonActivity = autoclass("org.kivy.android.PythonActivity")

context = PythonActivity.mActivity

event_success_listener = None
event_error_listener = None


class OnSuccessListener(PythonJavaClass):
    __javainterfaces__ = ["com/google/android/gms/tasks/OnSuccessListener"]
    __javacontext__ = "app"

    def __init__(self, success_listener):
        self.success_listener = success_listener

    @java_method("(Ljava/lang/Object;)V")
    def onSuccess(self, result):
        # user is signed in
        Logger.info("KivyAuth: Sign in successful using firebase.")
        user = FirebaseAuth.getInstance().getCurrentUser()

        _call_success(user)


class OnFailureListener(PythonJavaClass):
    __javainterfaces__ = ["com/google/android/gms/tasks/OnFailureListener"]
    __javacontext__ = "app"

    def __init__(self, error_listener):
        self.error_listener = error_listener

    @java_method("(Ljava/lang/Exception;)V")
    def onFailure(self, e):
        # handle exception
        Logger.info("KivyAuth: Sign in using firebase failed")
        self.error_listener()


def _call_success(user):
    event_success_listener(
        user.getDisplayName(),
        user.getEmail(),
        user.getPhotoUrl().toString() if user.getPhotoUrl() else ''
    )


def initialize_firebase(success_listener, error_listener):
    """
    Function to initialize firebase login.
    Must be called inside `build` method of kivy App before actual login.

    :param: `success_listener` - Function to be called on login success
    :param: `error_listener` - Function to be called on login error
    """
    FirebaseAuth.getInstance()
    Logger.info("KivyAuth: Initialized firebase auth")
    global event_success_listener
    event_success_listener = success_listener
    global event_error_listener
    event_error_listener = error_listener


def firebase_login(provider):
    pendingResultTask = FirebaseAuth.getPendingAuthResult()
    if pendingResultTask:
        # There's something already here! Finish the sign-in for your user.

        task = pendingResultTask.addOnSuccessListener(
            OnSuccessListener(event_success_listener)
        )
        task = task.addOnFailureListener(OnFailureListener(event_error_listener))
    else:
        # There's no pending result so you need to start the sign-in flow.

        task = FirebaseAuth.startActivityForSignInWithProvider(
            context, provider.build()
        )
        task = task.addOnSuccessListener(OnSuccessListener(event_success_listener))
        task = task.addOnFailureListener(OnFailureListener(event_error_listener))


def firebase_logout(after_logout):
    Logger.info("KivyAuth: Initiated logout using firebase")
    FirebaseAuth.getInstance().signOut()
    after_logout()
    Logger.info("KivyAuth: Logged out from firebase auth")


def auto_firebase():
    user = FirebaseAuth.getInstance().getCurrentUser()
    if user:
        _call_success(user)
        return True


def login_github():
    """
    Function to login using github
    """
    Logger.info("KivyAuth: Initiated github login")
    provider = OAuthProvider.newBuilder("github.com")
    firebase_login(provider)


def login_twitter():
    """
    Function to login using twitter
    """
    Logger.info("KivyAuth: Initiated twitter login")
    provider = OAuthProvider.newBuilder("twitter.com")
    firebase_login(provider)


def logout_twitter(after_logout):
    """
    Logout from twitter login

    :param: `after_logout` - Function to be called after logging out
    """
    firebase_logout(after_logout)


def logout_github(after_logout):
    """
    Logout from github login

    :param: `after_logout` - Function to be called after logging out
    """
    firebase_logout(after_logout)



import requests
from oauthlib.oauth2 import WebApplicationClient
import json
import webbrowser

from kivyauth.desktop.utils import (
    close_server,
    request,
    redirect,
    is_connected,
    start_server,
    app,
    _close_server_pls,
    port,
)
from kivy.app import App

# google configurations
GOOGLE_CLIENT_ID = ""
GOOGLE_CLIENT_SECRET = ""
GOOGLE_DISCOVERY_URL = "https://accounts.google.com/.well-known/openid-configuration"

client_google = None

event_success_listener = None
event_error_listener = None

__all__ = ("initialize_google", "login_google", "logout_google")


def get_google_provider_cfg():
    return requests.get(GOOGLE_DISCOVERY_URL).json()


def initialize_google(
    success_listener, error_listener, client_id=None, client_secret=None
):
    a = App.get_running_app()
    a.bind(on_stop=lambda *args: _close_server_pls(port))

    global event_success_listener
    event_success_listener = success_listener

    global event_error_listener
    event_error_listener = error_listener

    global GOOGLE_CLIENT_ID
    GOOGLE_CLIENT_ID = client_id

    global GOOGLE_CLIENT_SECRET
    GOOGLE_CLIENT_SECRET = client_secret

    global client_google
    client_google = WebApplicationClient(GOOGLE_CLIENT_ID)


@app.route("/loginGoogle")
def loginGoogle():
    # takeout auth endpoint url from google login
    google_provider_cfg = get_google_provider_cfg()

    authorization_endpoint = google_provider_cfg["authorization_endpoint"]
    # construct the request uri
    request_uri = client_google.prepare_request_uri(
        authorization_endpoint,
        redirect_uri=request.base_url + "/callbackGoogle",
        scope=["openid", "email", "profile"],
    )

    return redirect(request_uri)


@app.route("/loginGoogle/callbackGoogle")
def callbackGoogle():
    # Get authorization code Google sent back
    code = request.args.get("code")

    # Extract the URL to hit to get tokens
    # that allows to ask things on behalf of a user
    google_provider_cfg = get_google_provider_cfg()
    token_endpoint = google_provider_cfg["token_endpoint"]

    # prepare a request to get tokens
    token_url, headers, body = client_google.prepare_token_request(
        token_endpoint,
        authorization_response=request.url,
        redirect_url=request.base_url,
        code=code,
    )

    # send the request and get the response
    token_response = requests.post(
        token_url,
        headers=headers,
        data=body,
        auth=(GOOGLE_CLIENT_ID, GOOGLE_CLIENT_SECRET),
    )

    # parse the token response
    client_google.parse_request_body_response(json.dumps(token_response.json()))

    # Now we already have necessary tokens
    # lets ask Google for required informations
    userinfo_endpoint = google_provider_cfg["userinfo_endpoint"]
    uri, headers, body = client_google.add_token(userinfo_endpoint)

    userinfo_response = requests.get(uri, headers=headers, data=body).json()

    # parse the information
    if userinfo_response.get("email_verified"):
        close_server()

        event_success_listener(
            userinfo_response["name"],
            userinfo_response["email"],
            userinfo_response["picture"],
        )

        return "<h2>Success using google. Return to the application</h2>"

    event_error_listener()
    return "User Email not available or not verified"


def login_google():
    # print(App.get_running_app(), App.get_application_name)
    if is_connected():
        start_server(port)
        # from kivyauth.desktop.utils import func
        # App.get_running_app().bind(on_stop= lambda *args: func())

        webbrowser.open("https://127.0.0.1:{}/loginGoogle".format(port), 1, False)

    else:
        event_error_listener()


def logout_google(after_logout):
    """
    Logout from google login

    :param: `after_logout` - Function to be called after logging out
    """
    after_logout()



from kivy.logger import Logger
from kivyauth.android.firebase_auth import (
    firebase_login,
    firebase_logout,
    initialize_firebase,
    OAuthProvider,
    auto_firebase as auto_github,
)

__all__ = ("initialize_github", "login_github", "logout_github")


def initialize_github(succes_listener, error_listener, *args, **kwargs):
    initialize_firebase(succes_listener, error_listener)


def login_github():
    """
    Function to login using github
    """
    Logger.info("KivyAuth: Initiated github login")
    provider = OAuthProvider.newBuilder("github.com")
    firebase_login(provider)


def logout_github(after_logout):
    """
    Logout from github login

    :param: `after_logout` - Function to be called after logging out
    """
    firebase_logout(after_logout)



from flask import Flask, request, redirect
import requests
import os
import threading
import socket
from kivy.clock import Clock
import random

# # twitter configuration
# TWITTER_CLIENT_ID = ""
# TWITTER_CLIENT_SECRET = ""
# tw_authorization_endpoint = "https://api.twitter.com/oauth/authorize"
# tw_token_endpoint = "https://api.twitter.com/oauth/access_token"
# tw_userinfo_endpoint = "https://api.twitter.com/user"
# tw_request_token_endpoint = "https://api.twitter.com/oauth/request_token"

app = Flask(__name__)
app.secret_key = os.urandom(26)

PATH = os.path.dirname(__file__)
cert_path = os.path.join(PATH, "cert.pem")
key_path = os.path.join(PATH, "key.pem")

port = 9004
ran_num = random.randint(1111, 9999)


def _start_server(*args):
    try:
        app.run(host="127.0.0.1", port=port, ssl_context=(cert_path, key_path))
    except OSError:
        pass


def start_server(port):
    thread = threading.Thread(target=_start_server)

    thread.start()


@app.route("/kill{}".format(ran_num))
def close_server(*args, **kwargs):
    func = request.environ.get("werkzeug.server.shutdown")
    if func is None:
        raise RuntimeError("Not running with the Werkzeug Server")
    func()

    return ""


def stop_login(*args):
    _close_server_pls(port)


def _close_server_pls(port, *args):
    try:
        requests.get("https://127.0.0.1:{}/kill{}".format(port, ran_num), verify=False)
    except requests.exceptions.ConnectionError:
        pass


def is_connected():
    try:
        socket.create_connection(("www.google.com", 80))
        return True
    except OSError:
        pass
    return False


# @app.route("/loginTwitter")
# def loginTwitter():

#     # construct the request uri
#     # request_uri = client_twitter.prepare_request_uri(
#     #     tw_authorization_endpoint,
#     #     redirect_uri=request.base_url + "/callbackTwitter",
#     # )
#     # return redirect(request_uri)

#     resp = requests.post(
#         tw_request_token_endpoint,
#         {
#             "oauth_callback": "https%3A%2F%2F127.0.0.1%3A9004%2FloginTwitter%2FcallbackTwitter"
#         },
#     )
#     return resp.json()


# @app.route("/loginTwitter/callbackTwitter")
# def callbackTwitter():
#     code = request.args.get("code")

#     token_url, headers, body = client_twitter.prepare_token_request(
#         tw_token_endpoint,
#         client_id=TWITTER_CLIENT_ID,
#         client_secret=TWITTER_CLIENT_SECRET,
#         code=code,
#         redirect_url=request.base_url,
#     )

#     # send the request and get the response
#     token_response = requests.post(token_url, headers=headers, data=body)

#     # for some reason, token_response.json() is raising an error. So gotta do it manually
#     token_info = re.search(
#         "^access_token=(.*)&scope=.*&token_type=(.*)$", token_response.text
#     )
#     uri = tw_userinfo_endpoint
#     headers = {"Authorization": token_info.group(2) + " " + token_info.group(1)}
#     body = None

#     global userinfo_response
#     userinfo_response = requests.get(uri, headers=headers, data=body).json()

#     # parse the information
#     if userinfo_response.get("id"):
#         close_server()
#         print(get_userinfo())
#         print(get_userinfo())
#         # return update_database(
#         #     tmp["id"], tmp["email"], tmp["picture"]["data"]["url"], tmp["name"]
#         # )
#         return "Success using twitter. Return to the app"
#     return "User Email not available or not verified", 400


# if __name__ == "__main__":

#     if is_connected():
#         port = 9004
#         start_server(port)

#         webbrowser.open("https://127.0.0.1:{}/loginGoogle".format(port), 1, False)

#     else:
#         print("Could not connect. Check connection and try again")



from kivy.utils import platform

if platform == "android":

    def stop_login(*args):
        pass


elif platform != "ios":
    from kivyauth.desktop.utils import stop_login


class LoginProviders:
    google = "google"
    facebook = "facebook"
    github = "github"
    twitter = "twitter"


login_providers = LoginProviders()


def auto_login(provider):
    """
    Auto login using a given provider. You may call it `on_start`.

    :param: `provider` is one of `kivyauth.login_providers`
    """
    if platform == "android":
        if provider == login_providers.google:
            from kivyauth.android.google_auth import auto_google

            return auto_google()

        if provider == login_providers.facebook:
            from kivyauth.android.facebook_auth import auto_facebook

            return auto_facebook()

        if provider == login_providers.github:
            from kivyauth.android.github_auth import auto_github

            return auto_github()

        if provider == login_providers.twitter:
            from kivyauth.android.twitter_auth import auto_twitter

            return auto_twitter()

    else:
        raise NotImplementedError("Not yet availabe for desktop")



from kivy.logger import Logger
from kivyauth.android.firebase_auth import (
    firebase_login,
    firebase_logout,
    initialize_firebase,
    OAuthProvider,
    auto_firebase as auto_twitter,
)

__all__ = ("initialize_twitter", "login_twitter", "logout_twitter")


def initialize_twitter(succes_listener, error_listener, *args, **kwargs):
    initialize_firebase(succes_listener, error_listener)


def login_twitter():
    """
    Function to login using twitter
    """
    Logger.info("KivyAuth: Initiated twitter login")
    provider = OAuthProvider.newBuilder("twitter.com")
    firebase_login(provider)


def logout_twitter(after_logout):
    """
    Logout from twitter login

    :param: `after_logout` - Function to be called after logging out
    """
    firebase_logout(after_logout)



from kivy.logger import Logger
from kivy.utils import platform

__version__ = "2.3.2"
_log_message = "KivyAuth:" + f" {__version__}" + f' (installed at "{__file__}")'

__all__ = ("login_providers", "auto_login")

Logger.info(_log_message)



import requests
from oauthlib.oauth2 import WebApplicationClient
import json
import webbrowser
import random
import re

from kivyauth.desktop.utils import (
    close_server,
    request,
    redirect,
    is_connected,
    start_server,
    app,
    _close_server_pls,
    port,
)
from kivy.app import App

# github configuration
GITHUB_CLIENT_ID = ""
GITHUB_CLIENT_SECRET = ""
git_authorization_endpoint = "https://github.com/login/oauth/authorize"
git_token_endpoint = "https://github.com/login/oauth/access_token"
git_userinfo_endpoint = "https://api.github.com/user"

client_github = None

event_success_listener = None
event_error_listener = None

__all__ = ("initialize_github", "login_github", "logout_github")


def initialize_github(
    success_listener, error_listener, client_id=None, client_secret=None
):
    a = App.get_running_app()
    a.bind(on_stop=lambda *args: _close_server_pls(port))

    global event_success_listener
    event_success_listener = success_listener

    global event_error_listener
    event_error_listener = error_listener

    global GITHUB_CLIENT_ID
    GITHUB_CLIENT_ID = client_id

    global GITHUB_CLIENT_SECRET
    GITHUB_CLIENT_SECRET = client_secret

    global client_github
    client_github = WebApplicationClient(GITHUB_CLIENT_ID)


@app.route("/loginGithub")
def loginGithub():
    request_uri = client_github.prepare_request_uri(
        git_authorization_endpoint,
        redirect_uri=request.base_url + "/callbackGithub",
        scope=["read:user:email"],
        state="".join([random.choice("abcdefghijklmnopqrstuvwxyz") for _ in range(20)]),
    )
    return redirect(request_uri)


@app.route("/loginGithub/callbackGithub")
def callbackGithub():
    # Get authorization code Github sent back
    code = request.args.get("code")

    # prepare a request to get tokens
    token_url, headers, body = client_github.prepare_token_request(
        git_token_endpoint,
        client_id=GITHUB_CLIENT_ID,
        client_secret=GITHUB_CLIENT_SECRET,
        code=code,
        redirect_url=request.base_url,
        state="".join([random.choice("abcdefghijklmnopqrstuvwxyz") for _ in range(20)]),
    )

    # send the request and get the response
    token_response = requests.post(token_url, headers=headers, data=body)

    # for some reason, token_response.json() is raising an error. So gotta do it manually
    token_info = re.search(
        "^access_token=(.*)&scope=.*&token_type=(.*)$", token_response.text
    )

    headers = {"Authorization": token_info.group(2) + " " + token_info.group(1)}
    body = None

    userinfo_response = requests.get(
        git_userinfo_endpoint, headers=headers, data=body
    ).json()

    # parse the information
    if userinfo_response.get("id"):
        close_server()
        event_success_listener(
            userinfo_response["name"],
            userinfo_response["email"],
            userinfo_response["avatar_url"],
        )
        return "<h2>Success using Github. Return to the application</h2>"

    event_error_listener()
    return "User Email not available or not verified"


def login_github():
    if is_connected():
        port = 9004
        start_server(port)

        webbrowser.open("https://127.0.0.1:{}/loginGithub".format(port), 1, False)

    else:
        event_error_listener()


def logout_github(after_logout):
    """
    Logout from github login

    :param: `after_logout` - Function to be called after logging out
    """
    after_logout()



from kivy.utils import platform

if platform == "android":
    from kivyauth.android.twitter_auth import (
        initialize_twitter,
        login_twitter,
        logout_twitter,
    )

elif platform != "ios":
    from kivyauth.desktop.twitter_auth import (
        initialize_twitter,
        login_twitter,
        logout_twitter,
    )



from kivy.utils import platform

if platform == "android":
    from kivyauth.android.facebook_auth import (
        initialize_fb,
        login_facebook,
        logout_facebook,
    )

elif platform != "ios":
    from kivyauth.desktop.facebook_auth import (
        initialize_fb,
        login_facebook,
        logout_facebook,
    )



from android.activity import bind as result_bind
from jnius import autoclass
from kivy.clock import mainthread
from kivy.logger import Logger

Gso = autoclass("com.google.android.gms.auth.api.signin.GoogleSignInOptions")
GsoBuilder = autoclass(
    "com.google.android.gms.auth.api.signin.GoogleSignInOptions$Builder"
)
GSignIn = autoclass("com.google.android.gms.auth.api.signin.GoogleSignIn")
ApiException = autoclass("com.google.android.gms.common.api.ApiException")
PythonActivity = autoclass("org.kivy.android.PythonActivity")
context = PythonActivity.mActivity

RC_SIGN_IN = 10122
mGSignInClient = None
event_success_listener = None

__all__ = ("initialize_google", "login_google", "logout_google")


class GoogleActivityListener:
    def __init__(self, success_listener, error_listener):
        self.success_listener = success_listener
        self.error_listener = error_listener

    def google_activity_listener(self, request_code, result_code, data):
        if request_code == RC_SIGN_IN:
            Logger.info("KivyAuth: google_activity_listener called.")
            task = GSignIn.getSignedInAccountFromIntent(data)
            try:
                account = task.getResult(ApiException)
                _call_success(account)

            except Exception as e:
                Logger.info("KivyAuth: Error signing in using Google. {}".format(e))
                self.error_listener()


def _call_success(account):
    if account:
        Logger.info("KivyAuth: Google Login success. Calling success listener.")
        event_success_listener(
            account.getDisplayName(),
            account.getEmail(),
            account.getPhotoUrl().toString() if account.getPhotoUrl() else '',
        )
        return True


def initialize_google(success_listener, error_listener, *args, **kwargs):
    """
    Function to initialize google login.
    Must be called inside `build` method of kivy App before actual login.

    :param: `success_listener` - Function to be called on login success
    :param: `error_listener` - Function to be called on login error
    """
    global event_success_listener
    event_success_listener = success_listener
    gso = GsoBuilder(Gso.DEFAULT_SIGN_IN).requestEmail().build()
    global mGSignInClient
    mGSignInClient = GSignIn.getClient(context, gso)
    gal = GoogleActivityListener(success_listener, error_listener)
    result_bind(on_activity_result=gal.google_activity_listener)

    Logger.info("KivyAuth: Initialized google signin")


# @mainthread
def login_google():
    """
    Function to login using google
    """
    Logger.info("KivyAuth: Initiated google login")
    signInIntent = mGSignInClient.getSignInIntent()
    context.startActivityForResult(signInIntent, RC_SIGN_IN)


def auto_google():
    """
    Auto login using Google. You may call it `on_start`.
    """
    account = GSignIn.getLastSignedInAccount(context)
    return _call_success(account)


def logout_google(after_logout):
    """
    Logout from google login

    :param: `after_logout` - Function to be called after logging out
    """
    mGSignInClient.signOut()
    after_logout()
    Logger.info("KivyAuth: Logged out from google login")



from kivy.utils import platform

if platform == "android":
    from kivyauth.android.google_auth import (
        initialize_google,
        login_google,
        logout_google,
    )

elif platform != "ios":
    from kivyauth.desktop.google_auth import (
        initialize_google,
        login_google,
        logout_google,
    )



from kivy.utils import platform

if platform == "android":
    from kivyauth.android.github_auth import (
        initialize_github,
        login_github,
        logout_github,
    )

elif platform != "ios":
    from kivyauth.desktop.github_auth import (
        initialize_github,
        login_github,
        logout_github,
    )



from setuptools import setup
import os, re

with open("README.md", "r") as fh:
    long_description = fh.read()


def is_android():
    if "ANDROID_BOOTLOGO" in os.environ:
        return True
    return False


def get_version() -> str:
    """Get __version__ from __init__.py file."""
    version_file = os.path.join(os.path.dirname(__file__), "kivyauth", "__init__.py")
    version_file_data = open(version_file, "rt", encoding="utf-8").read()
    version_regex = r"(?<=^__version__ = ['\"])[^'\"]+(?=['\"]$)"
    try:
        version = re.findall(version_regex, version_file_data, re.M)[0]
        return version
    except IndexError:
        raise ValueError(f"Unable to find version string in {version_file}.")


setup(
    name="KivyAuth",
    version=get_version(),
    packages=["kivyauth"],
    package_data={"kivyauth": ["*.py", "desktop/*", "android/*"],},
    # metadata to display on PyPI
    author="Shashi Ranjan",
    author_email="shashiranjankv@gmail.com",
    description="Integrate Google, Facebook, Github & Twitter login in kivy applications ",
    long_description=long_description,
    long_description_content_type="text/markdown",
    keywords="social-login google-login facebook-login firebase-auth kivy-application kivy python",
    url="https://github.com/shashi278/social-auth-kivy",
    classifiers=[
        "Programming Language :: Python :: 3",
        "License :: OSI Approved :: MIT License",
        "Operating System :: Android",
        "Operating System :: Microsoft :: Windows",
        "Operating System :: OS Independent"
    ],
    install_requires=["kivy>=2.0.0", "oauthlib", "flask", "requests"]
    if not is_android()
    else [],
    python_requires=">=3.6",
)



from android.activity import bind as result_bind
from jnius import PythonJavaClass, autoclass, cast, java_method
from kivy.logger import Logger

CallbackManagerFactory = autoclass("com.facebook.CallbackManager$Factory")
FacebookSdk = autoclass("com.facebook.FacebookSdk")
LoginManager = autoclass("com.facebook.login.LoginManager")
AccessToken = autoclass("com.facebook.AccessToken")
GraphRequest = autoclass("com.facebook.GraphRequest")
ImageRequest = autoclass("com.facebook.internal.ImageRequest")

Bundle = autoclass("android.os.Bundle")
Arrays = autoclass("java.util.Arrays")

PythonActivity = autoclass("org.kivy.android.PythonActivity")

context = PythonActivity.mActivity
mLoginMgr = None
mList = None


__all__ = ("initialize_fb", "login_facebook", "logout_facebook")


class PythonGraphJSONObjectCallback(PythonJavaClass):
    __javainterfaces__ = ["com/facebook/GraphRequest$GraphJSONObjectCallback"]
    __javacontext__ = "app"

    def __init__(self, complete_listener):
        self.complete_listener = complete_listener
        super().__init__()

    @java_method("(Lorg/json/JSONObject;Lcom/facebook/GraphResponse;)V")
    def onCompleted(self, me, response):
        if response.getError():
            # handle error
            Logger.error("KivyAuth: Unable to retrieve profile info")

        else:

            if AccessToken.isCurrentAccessTokenActive():
                access_token = AccessToken.getCurrentAccessToken().getToken()
            else:
                access_token = ""

            uri = ImageRequest.getProfilePictureUri(
                me.optString("id"), 200, 200, access_token
            )

            Logger.info(
                "KivyAuth: Profile info retrieved successfully."
                " Calling success listener."
            )

            self.complete_listener(
                me.optString("first_name") + " " + me.optString("last_name"),
                me.optString("email"),
                uri.toString() if uri else '',
            )


class PythonFacebookCallback(PythonJavaClass):
    __javainterfaces__ = ["com/facebook/FacebookCallback"]
    __javacontext__ = "app"

    def __init__(self, success_listener, cancel_listener, error_listener):
        self.success_listener = success_listener
        self.cancel_listener = cancel_listener
        self.error_listener = error_listener

    @java_method("(Ljava/lang/Object;)V")
    def onSuccess(self, result):
        Logger.info("KivyAuth: Login success. Requesting profile info.")

        request = GraphRequest.newMeRequest(
            result.getAccessToken(),
            PythonGraphJSONObjectCallback(self.success_listener),
        )

        params = Bundle()
        params.putString("fields", "last_name,first_name,email")
        request.setParameters(params)
        request.executeAsync()

    @java_method("()V")
    def onCancel(self):
        Logger.info("KivyAuth: Login Cancelled.")
        self.cancel_listener()

    @java_method("(Lcom/facebook/FacebookException;)V")
    def onError(self, error):
        Logger.error("KivyAuth: Error logging in.")
        self.error_listener()


def initialize_fb(success_listener, error_listener, *args, **kwargs):
    """
    Function to initialize facebook login.
    Must be called inside `build` method of kivy App before actual login.

    :param: `success_listener` - Function to be called on login success
    :param: `error_listener` - Function to be called on login error
    """
    FacebookSdk.sdkInitialize(context.getApplicationContext())
    mCallbackManager = CallbackManagerFactory.create()
    mFacebookCallback = PythonFacebookCallback(
        success_listener, error_listener, error_listener
    )
    result_bind(on_activity_result=mCallbackManager.onActivityResult)

    Logger.info("KivyAuth: Initialized facebook signin")
    global mList
    mList = [mCallbackManager, mFacebookCallback]


def login_facebook():
    """
    Function to login using facebook
    """
    Logger.info("KivyAuth: Initiated facebook login")
    global mLoginMgr
    mLoginMgr = LoginManager.getInstance()
    mLoginMgr.registerCallback(*mList)
    mLoginMgr.logInWithReadPermissions(
        cast(autoclass("android.app.Activity"), context),
        Arrays.asList("email", "public_profile"),
    )


def auto_facebook():
    """
    Auto login using Facebook. You may call it `on_start`.
    """
    accessToken = AccessToken.getCurrentAccessToken()
    if accessToken and not accessToken.isExpired():
        login_facebook()
        return True


def logout_facebook(after_logout):
    """
    Logout from facebook login

    :param: `after_logout` - Function to be called after logging out
    """
    mLoginMgr.logOut()
    after_logout()
    Logger.info("KivyAuth: Logged out from facebook login")



import requests
from oauthlib.oauth2 import WebApplicationClient
import json
import webbrowser
import random

from kivyauth.desktop.utils import (
    close_server,
    request,
    redirect,
    is_connected,
    start_server,
    app,
    _close_server_pls,
    port,
)
from kivy.app import App

# facebook configuration
FACEBOOK_CLIENT_ID = ""
FACEBOOK_CLIENT_SECRET = ""
fb_authorization_endpoint = "https://www.facebook.com/v4.0/dialog/oauth?"
fb_token_endpoint = "https://graph.facebook.com/v4.0/oauth/access_token?"
fb_userinfo_endpoint = "https://graph.facebook.com/v4.0/me?"

client_facebook = None

event_success_listener = None
event_error_listener = None

__all__ = ("initialize_fb", "login_facebook", "logout_facebook")


def initialize_fb(success_listener, error_listener, client_id=None, client_secret=None):
    a = App.get_running_app()
    a.bind(on_stop=lambda *args: _close_server_pls(port))

    global event_success_listener
    event_success_listener = success_listener

    global event_error_listener
    event_error_listener = error_listener

    global FACEBOOK_CLIENT_ID
    FACEBOOK_CLIENT_ID = client_id

    global FACEBOOK_CLIENT_SECRET
    FACEBOOK_CLIENT_SECRET = client_secret

    global client_facebook
    client_facebook = WebApplicationClient(FACEBOOK_CLIENT_ID)


@app.route("/loginFacebook")
def loginFacebook():

    st = "".join([random.choice("abcdefgh1234567") for _ in range(10)])
    ds = "".join([random.choice("1234567890") for _ in range(10)])

    request_uri = client_facebook.prepare_request_uri(
        fb_authorization_endpoint,
        redirect_uri=request.base_url + "/callbackFacebook",
        scope=["email"],
        state="{st=" + st + ",ds=" + ds + "}",
    )

    return redirect(request_uri)


@app.route("/loginFacebook/callbackFacebook")
def callbackFacebook():
    code = request.args.get("code")

    # prepare a request to get tokens
    token_url, headers, body = client_facebook.prepare_token_request(
        fb_token_endpoint,
        client_id=FACEBOOK_CLIENT_ID,
        client_secret=FACEBOOK_CLIENT_SECRET,
        code=code,
        redirect_url=request.base_url,
    )

    # send the request and get the response
    token_response = requests.post(token_url, headers=headers, data=body)

    token_url = (
        fb_token_endpoint
        + "client_id={}"
        + "&client_secret={}"
        + "&grant_type=client_credentials"
    ).format(FACEBOOK_CLIENT_ID, FACEBOOK_CLIENT_SECRET)

    # send the request and get the response
    # app_token_response = requests.get(token_url, headers=headers, data=body)

    headers = {
        "Authorization": token_response.json()["access_token"]
        + " "
        + token_response.json()["token_type"]
    }

    request_uri = client_facebook.prepare_request_uri(
        fb_userinfo_endpoint,
        fields=["id", "name", "email", "picture"],
        access_token=token_response.json()["access_token"],
    )

    # make the request and get the response
    userinfo_response = requests.get(request_uri, headers=headers, data=None).json()

    # parse the information
    if userinfo_response.get("id"):
        close_server()
        event_success_listener(
            userinfo_response["name"],
            userinfo_response["email"],
            userinfo_response["picture"]["data"]["url"],
        )
        return "<h2>Success using facebook. Return to the app</h2>"

    event_error_listener()
    return "User Email not available or not verified"


def login_facebook():
    if is_connected():
        port = 9004
        start_server(port)

        webbrowser.open("https://127.0.0.1:{}/loginFacebook".format(port), 1, False)

    else:
        event_error_listener()


def logout_facebook(after_logout):
    """
    Logout from facebook login

    :param: `after_logout` - Function to be called after logging out
    """
    after_logout()
